package cn.ibizlab.humanresource.core.base.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.base.domain.HROMHierarchy;
import cn.ibizlab.humanresource.core.base.filter.HROMHierarchySearchContext;
import cn.ibizlab.humanresource.core.base.service.IHROMHierarchyService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.base.mapper.HROMHierarchyMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[组织层次结构] 服务对象接口实现
 */
@Slf4j
@Service("HROMHierarchyServiceImpl")
public class HROMHierarchyServiceImpl extends ServiceImpl<HROMHierarchyMapper, HROMHierarchy> implements IHROMHierarchyService {


    protected cn.ibizlab.humanresource.core.base.service.IHROMHierarchyService hromhierarchyService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROMHierarchyCatService hromhierarchycatService;
    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HROMHierarchy et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getOmhierarchyid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HROMHierarchy> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HROMHierarchy et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("omhierarchyid",et.getOmhierarchyid())))
            return false;
        CachedBeanCopier.copy(get(et.getOmhierarchyid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HROMHierarchy> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HROMHierarchy get(String key) {
        HROMHierarchy et = getById(key);
        if(et==null){
            et=new HROMHierarchy();
            et.setOmhierarchyid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HROMHierarchy getDraft(HROMHierarchy et) {
        return et;
    }

    @Override
    public boolean checkKey(HROMHierarchy et) {
        return (!ObjectUtils.isEmpty(et.getOmhierarchyid()))&&(!Objects.isNull(this.getById(et.getOmhierarchyid())));
    }
    @Override
    @Transactional
    public boolean save(HROMHierarchy et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HROMHierarchy et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HROMHierarchy> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HROMHierarchy> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HROMHierarchy> selectByHromhierarchycatid(String omhierarchycatid) {
        return baseMapper.selectByHromhierarchycatid(omhierarchycatid);
    }

    @Override
    public void removeByHromhierarchycatid(String omhierarchycatid) {
        this.remove(new QueryWrapper<HROMHierarchy>().eq("hromhierarchycatid",omhierarchycatid));
    }

	@Override
    public List<HROMHierarchy> selectByPomhierarchyid(String omhierarchyid) {
        return baseMapper.selectByPomhierarchyid(omhierarchyid);
    }

    @Override
    public void removeByPomhierarchyid(String omhierarchyid) {
        this.remove(new QueryWrapper<HROMHierarchy>().eq("pomhierarchyid",omhierarchyid));
    }

	@Override
    public List<HROMHierarchy> selectByOrganizationid(String organizationid) {
        return baseMapper.selectByOrganizationid(organizationid);
    }

    @Override
    public void removeByOrganizationid(String organizationid) {
        this.remove(new QueryWrapper<HROMHierarchy>().eq("organizationid",organizationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HROMHierarchy> searchDefault(HROMHierarchySearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HROMHierarchy> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HROMHierarchy>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HROMHierarchy> getHromhierarchyByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HROMHierarchy> getHromhierarchyByEntities(List<HROMHierarchy> entities) {
        List ids =new ArrayList();
        for(HROMHierarchy entity : entities){
            Serializable id=entity.getOmhierarchyid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



