package cn.ibizlab.humanresource.core.humanresource.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.humanresource.core.humanresource.domain.HREmployee;
import cn.ibizlab.humanresource.core.humanresource.filter.HREmployeeSearchContext;
import cn.ibizlab.humanresource.core.humanresource.service.IHREmployeeService;

import cn.ibizlab.humanresource.util.helper.CachedBeanCopier;
import cn.ibizlab.humanresource.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.humanresource.core.humanresource.mapper.HREmployeeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[员工] 服务对象接口实现
 */
@Slf4j
@Service("HREmployeeServiceImpl")
public class HREmployeeServiceImpl extends ServiceImpl<HREmployeeMapper, HREmployee> implements IHREmployeeService {

    @Autowired
    @Lazy
    protected cn.ibizlab.humanresource.core.base.service.IHROrganizationService hrorganizationService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(HREmployee et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEmployeeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<HREmployee> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(HREmployee et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("employeeid",et.getEmployeeid())))
            return false;
        CachedBeanCopier.copy(get(et.getEmployeeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<HREmployee> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public HREmployee get(String key) {
        HREmployee et = getById(key);
        if(et==null){
            et=new HREmployee();
            et.setEmployeeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public HREmployee getDraft(HREmployee et) {
        return et;
    }

    @Override
    public boolean checkKey(HREmployee et) {
        return (!ObjectUtils.isEmpty(et.getEmployeeid()))&&(!Objects.isNull(this.getById(et.getEmployeeid())));
    }
    @Override
    @Transactional
    public boolean save(HREmployee et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(HREmployee et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<HREmployee> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<HREmployee> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<HREmployee> selectByOrganizationid(String organizationid) {
        return baseMapper.selectByOrganizationid(organizationid);
    }

    @Override
    public void removeByOrganizationid(String organizationid) {
        this.remove(new QueryWrapper<HREmployee>().eq("organizationid",organizationid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<HREmployee> searchDefault(HREmployeeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<HREmployee> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<HREmployee>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<HREmployee> getHremployeeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<HREmployee> getHremployeeByEntities(List<HREmployee> entities) {
        List ids =new ArrayList();
        for(HREmployee entity : entities){
            Serializable id=entity.getEmployeeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



